#include "testlib.h"
#include <string>
#include <vector>
#include <sstream>
#include <iostream>
#include <ctime>
#include <set>

using namespace std;

FILE *flog;

bool WS(char c)
{
	return (c == ' ') || (c == 10) || (c == 13) || (c == '\t');
}


const int SZ = 15;
const int di[4] = {0, -1, 0, 1};
const int dj[4] = {-1, 0, 1, 0};

void PlayerWinF(int player, const char * format, ...)
{
    char * buffer = new char [MAX_FORMAT_BUFFER_SIZE];
    
    va_list ap;
    va_start(ap, format);
    std::vsprintf(buffer, format, ap);
    va_end(ap);

    std::string output(buffer);
    delete[] buffer;

    if (player == 1) printf("First Wins\n");
    else if (player == 2) printf("Second Wins\n");
    else printf("Draw\n");
    printf("%s\n\n", output.c_str());
    exit(0);
}

char field[SZ][SZ + 1];
int curpl, curmove;

void WriteField(int curpl)
{
	int i, j;
	if (curpl == 1)
	{
		for (i = 0; i < SZ; i++)
		{
			for (j = 0; j < SZ; j++)
			{
				printf("%c", field[i][j]);
			}
			printf("\n");
		}
	}
	else
	{
		for (i = SZ - 1; i >= 0; i--)
		{
			for (j = SZ - 1; j >= 0; j--)
			{
				if (field[i][j] == '1') printf("2");
				if (field[i][j] == '2') printf("1");
				if (field[i][j] == '3') printf("4");
				if (field[i][j] == '4') printf("3");
				if (field[i][j] == '.') printf(".");
			}
			printf("\n");
		}
	}
	printf("\n");
	fflush(stdout);
}

int GetDNUM(char c)
{
	if (c == 'L') return 0;
	if (c == 'R') return 2;
	if (c == 'U') return 1;
	if (c == 'D') return 3;
	return -1;
}

set<pair<int, int> > was;
vector<pair<int, int> > q;

void RePaint(int curpl)
{
	q.clear();
	was.clear();
	q.push_back(make_pair(-1, -1));
	was.insert(q[0]);
	int hd = 0;
	while (hd < q.size())
	{
		int ci, cj, ni, nj, k;
		ci = q[hd].first;
		cj = q[hd].second;
		for (k = 0; k < 4; k++)
		{
			ni = ci + di[k];
			nj = cj + dj[k];
			if (ni < -1 || nj < -1 || ni > SZ || nj > SZ) continue;
			if (was.find(make_pair(ni, nj)) != was.end()) continue;
			if (ni >= 0 && ni < SZ && nj >= 0 && nj <= SZ)
			{
				if (field[ni][nj] == curpl + '1' - 1) continue;
				if (field[ni][nj] == curpl + '3' - 1) continue;
			}
			q.push_back(make_pair(ni, nj));
			was.insert(make_pair(ni, nj));
		}
		hd++;
	}
	int i, j;
	for (i = 0; i < SZ; i++)
	{
		for (j = 0; j < SZ; j++)
		{
			if (field[i][j] == '.' && was.find(make_pair(i, j)) == was.end())
			{
				field[i][j] = '1' + curpl - 1;
			}
		}
	}
}

void ProcessMove(int curpl, int k)
{
	int bi, bj, i, j;
	bi = bj = 0;
	for (i = 0; i < SZ; i++)
	{
		for (j = 0; j < SZ; j++)
		{
			if (field[i][j] == '3' + curpl - 1)
			{
				bi = i;
				bj = j;
			}
		}
	}
	int ni, nj;
	ni = bi + di[k];
	nj = bj + dj[k];
	if (ni < 0 || nj < 0 || ni >= SZ || nj >= SZ) return;
	if (field[ni][nj] != '1' + curpl - 1 && field[ni][nj] != '.') return;
	field[ni][nj] = '3' + curpl - 1;
	field[bi][bj] = '1' + curpl - 1;
	// repaint field
	RePaint(curpl);
}

bool GameFinished()
{
	int i, j;
	for (i = 0; i < SZ; i++)
	{
		for (j = 0; j < SZ; j++)
		{
			if (field[i][j] == '.') return false;
		}
	}
	return true;
}

int main(int argc, char * argv[])
{
	setbuf(stdin, NULL);
    setName("checker for xonix game problem");
    inf.init(stdin, _input);

    flog = fopen("game.log", "wt");

    strcpy(field[0],  "3..............");
    strcpy(field[1],  "...............");
    strcpy(field[2],  "...............");
    strcpy(field[3],  "...............");
    strcpy(field[4],  "...............");
    strcpy(field[5],  "...............");
    strcpy(field[6],  "...............");
    strcpy(field[7],  "...............");
    strcpy(field[8],  "...............");
    strcpy(field[9],  "...............");
    strcpy(field[10], "...............");
    strcpy(field[11], "...............");
    strcpy(field[12], "...............");
    strcpy(field[13], "...............");
    strcpy(field[14], "..............4");

    int i, j;
    curpl = 1;
	curmove = 0;
    while (curmove < 1000)
    {
    	if (curmove > 0) printf("Move OK\n");
    	WriteField(curpl);
    	for (i = 0; i < SZ; i++)
    	{
    		for (j = 0; j < SZ; j++)
    		{
    			fprintf(flog, "%c", field[i][j]);
    		}
    		fprintf(flog, "\n");
    	}
    	fflush(flog);
    	char c;
		c = inf.readChar();
		while (WS(c))
		{
			c = inf.readChar();
		}
		if (!((c == 'L') || (c == 'U') || (c == 'D') || (c == 'R')))
		{
    		PlayerWinF(3 - curpl, "Player %d does not match output format", curpl);
		}
		inf.nextLine();
		inf.nextLine();
		// process move
		int k = GetDNUM(c);
		if (curpl == 2) k = (k + 2) % 4;
		ProcessMove(curpl, k);
		//
		if (GameFinished() || curmove == 999) break;
    	curpl = 3 - curpl;
    	curmove++;
    }
    int p1score, p2score;
    p1score = p2score = 0;
    for (i = 0; i < SZ; i++)
    {
    	for (j = 0; j < SZ; j++)
    	{
    		fprintf(flog, "%c", field[i][j]);
    		if (field[i][j] == '1' || field[i][j] == '3') p1score++;
    		if (field[i][j] == '2' || field[i][j] == '4') p2score++;
    	}
    	fprintf(flog, "\n");
    }
    if (p1score > p2score) PlayerWinF(1, "Player1: %d Player2: %d", p1score, p2score);
    else if (p1score < p2score) PlayerWinF(2, "Player1: %d Player2: %d", p1score, p2score);
    else PlayerWinF(0, "Player1: %d Player2: %d", p1score, p2score);
    return 0;
}
