#include "libproc.h"

#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>

using namespace std;

void RunProcess(string cmdline, string dir, int &pid, int &rd, int &wr)
{
	// create pipes for child process
	int readpipe[2]; // 0 - read 1 - write
	int writepipe[2];
	if (pipe(readpipe)) throw (string)"Unable to create pipe";
	if (pipe(writepipe)) throw (string)"Unable to create pipe";
	int res = fork();
	if (res == -1) throw (string)"Unable to fork";
	if (res == 0)
	{
		// we are child process
		close(readpipe[0]);
		close(writepipe[1]);
		dup2(readpipe[1], 1);
		dup2(writepipe[0], 0);
		// change to correct directory
		chdir(dir.c_str());
		// split command line into command and args
		int i;
		string cmd, args;
		for (i = 0; i <= cmdline.length(); i++)
		{
			if (i == cmdline.length() || cmdline[i] == ' ')
			{
				int j;
				cmd = "";
				for (j = 0; j < i; j++) cmd += cmdline[j];
				args = "";
				for (j = i + 1; j < cmdline.length(); j++) args += cmdline[j];
				break;
			}
		}
		//cerr << "trying to run " << cmd << "\n";
		//cerr << "args = " << args << "\n";
		execl(cmd.c_str(), cmd.c_str(), args.c_str(), (char*)NULL);
		// we are not in this code anymore from here :-)
		cerr << "run error, we should not be here...\n";
		cerr << "errno = " << errno << "\n";
	}
	else
	{
		// we are parent process
		pid = res;
		close(readpipe[1]);
		rd = readpipe[0];
		close(writepipe[0]);
		wr = writepipe[1];
		// need to switch pipe ends into nonblocking mode
		int flags = fcntl(rd, F_GETFL, 0);
		fcntl(rd, F_SETFL, flags | O_NONBLOCK);
		flags = fcntl(wr, F_GETFL, 0);
		fcntl(wr, F_SETFL, flags | O_NONBLOCK);
	}
}

double GetProcessTime(int pid)
{
	ostringstream oss;
	oss << "/proc/" << pid << "/stat";
	FILE *fproc = fopen(oss.str().c_str(), "rt");
	if (!fproc) return 0;
	unsigned int ut, kt;
	fscanf(fproc, "%*d %*s %*c %*d %*d %*d %*d %*d %*u %*lu %*lu %*lu %*lu %lu %lu", &ut, &kt);
	fclose(fproc);
	return ((double)ut + kt) / ((double)sysconf(_SC_CLK_TCK));
}


long long GetProcessMemory(int pid)
{
	ostringstream oss;
	oss << "/proc/" << pid << "/stat";
	FILE *fproc = fopen(oss.str().c_str(), "rt");
	if (!fproc) return 0;
	unsigned int m;
	fscanf(fproc, "%*d %*s %*c %*d %*d %*d %*d %*d %*u %*lu %*lu %*lu %*lu %*lu %*lu %*ld %*ld %*ld %*ld %*ld %*ld %*llu %lu", &m);
	fclose(fproc);
	return m;
}
